' ============================================================================
' transcode.vbs - a transcoding script (not only for) Windows Home Server 
' Author: jensemann (jensemann@justgo.de)
' 
' Version: 0.1
'
' changelog:
' adapted version for multi transoding (avi to mpg & mp4)
' fix: toLower for file extension (.AVI .MPG will be recognized now)
' add: logging for each trancsoding
' 
' based on: http://wiki.videolan.org/Transcode
' 
' Video Codecs: http://wiki.videolan.org/Codec#Video_Codecs
' Audio Codecs: http://wiki.videolan.org/Codec#Audio_Codecs
' Muxers:       http://wiki.videolan.org/Codec#Muxers
'
' ============================================================================
' Todo: Integrate DVD to mp4 transcoding:
' % vlc -vvv dvd://[device][@raw_device][@[title][,[chapter][,angle]]]
'
' ============================================================================



' ------------ VLC Homedir -------------------
strVLCDir = "C:\Programme\VideoLAN\VLC\"

' ------------- path settings ----------------
strSource = "C:\in"
strTarget = "C:\out"



Dim objFSO, objShell, strSource, strTarget, strOldName, strNewName
set objFSO = CreateObject("Scripting.FileSystemObject")

Wscript.Echo "============== VLC Video Transcoding =================="
Wscript.Echo "" 
Wscript.Echo "Batch file video transcoding with vlc" 
Wscript.Echo "written by: Jensemann (jensemann@justgo.de)" 
Wscript.Echo "" 
Wscript.Echo "Config:" 
Wscript.Echo "Source Dir (=Input) : " & strSource
Wscript.Echo "Target Dir (=Output): " & strTarget
Wscript.Echo "VLC Homedir:          " & strVLCDir
Wscript.Echo "======================================================="
Wscript.Echo "" 
Wscript.Echo ""

Dim FilesFound, FileTranscoded, FileSkipped, FileIgnored
	
Do
	FilesFound = 0
	FileTranscoded = 0
	FileSkipped = 0
	FileIgnored = 0
	
	call RecurseDir(strSource, strTarget)
	'
	' Recursively transcode all files and folders from strSource to strTarget
	  Wscript.Echo Date() & " " & Time() & " done (" & FilesFound & " files found, " & FileTranscoded & " transcoded, " & FileSkipped& " skipped, " & FileIgnored & " ignored)"
	  
	Wscript.Echo "waiting.."
    intWarten = 600000 ' Millisec
    WScript.Sleep intWarten
Loop

Sub RecurseDir(strSource, strTarget)
	Dim strTempSource, strTempTarget
	Set strTempSource = objFSO.GetFolder(strSource)

	If Not objFSO.FolderExists(strTarget) Then
        Wscript.Echo "Creating folder [" & strTarget & "]"
		objFSO.CreateFolder(strTarget)
	End If

	Set strTempTarget = objFSO.GetFolder(strTarget)
	
	' ** transcode all files from current directory to target dir:
	For each file in strTempSource.files
		strOldName = strSource & "\" & file.name
    	Wscript.Echo "======================================================"
		Wscript.Echo Date() &" "& Time() &" "& strOldName & ".."
		Wscript.Echo ""
		
		' get file extension of original file
		strExt = LCase((Right(strOldName, 3)))
		
		FilesFound = FilesFound +1

		Select Case strExt
			Case ".ts"
		    	' --------------------- TS to mp4 Transcoding ---------------------------
				strNewName = strTempTarget & "\" & Mid(file.name, 1, InstrRev(file.name, ".")) & "mp4"
				If Not objFSO.FileExists(strNewName) Then
				    ' same transcoding like mpg source
					Call TranscodeMPG(strOldName, strNewName)					
				Else 
				   Wscript.Echo "WARN: Output file exist: " & strNewName & " (skipping)"
    			   FileSkipped=FileSkipped+1
				End If
									
			Case "mpeg"
			    ' --------------------- MPEG to MP4 Transcoding --------------------------
				strNewName = strTempTarget & "\" & Mid(file.name, 1, InstrRev(file.name, ".")) & "mp4"
				If Not objFSO.FileExists(strNewName) Then
					Call TranscodeMPG(strOldName, strNewName)
				Else 
				   Wscript.Echo "WARN: Output file exist: " & strNewName & " (skipping)"
    			   FileSkipped=FileSkipped+1
				End If
				
			Case "mpg"
			    ' --------------------- MPEG to MP4 Transcoding --------------------------
				strNewName = strTempTarget & "\" & Mid(file.name, 1, InstrRev(file.name, ".")) & "mp4"
				If Not objFSO.FileExists(strNewName) Then
					Call TranscodeMPG(strOldName, strNewName)
				Else 
				   Wscript.Echo "WARN: Output file exist: " & strNewName & " (skipping)"
    			   FileSkipped=FileSkipped+1
				End If
				

			Case "mp4"
			    ' --------------------- MP4 to MP4 (Web) Transcoding --------------------------
				strNewName = strTempTarget & "\" & Mid(file.name, 1, InstrRev(file.name, ".")) & "mp4"
				If Not objFSO.FileExists(strNewName) Then
					Call TranscodeMP4(strOldName, strNewName)
				Else 
				   Wscript.Echo "WARN: Output file exist: " & strNewName & " (skipping)"
    			   FileSkipped=FileSkipped+1
				End If

			Case "avi"
				' -------------- a placeholder for a new transcoding ---------------------------
				' double encoding for AVI input
				strNewName = strTempTarget & "\" & Mid(file.name, 1, InstrRev(file.name, ".")) & "mpg"
				If Not objFSO.FileExists(strNewName) Then
					Call TranscodeAVItoMPG2(strOldName, strNewName)
				Else 
				   Wscript.Echo "WARN: Output file exist: " & strNewName & " (skipping)"
    			   FileSkipped=FileSkipped+1
				End If
					
				strNewName = strTempTarget & "\" & Mid(file.name, 1, InstrRev(file.name, ".")) & "mp4"
				If Not objFSO.FileExists(strNewName) Then
					Call TranscodeAVItoMP4(strOldName, strNewName)
				Else 
				   Wscript.Echo "WARN: Output file exist: " & strNewName & " (skipping)"
    			   FileSkipped=FileSkipped+1
				End If
													
		    ' --------------- unknown extension (=warning) --------------------
			Case Else
			   wscript.echo "WARN: No transcoding for extension " & strExt  & " (ignored)"
   			   FileIgnored=FileIgnored+1
				
		End Select
		Wscript.Echo "======================================================"
 		Wscript.echo ""
 		Wscript.echo ""
	Next

	'  for each subfolder of current dir, transcode files:
	For each subdir in strTempSource.subfolders
  	call RecurseDir(subdir.path, strTempTarget & "\" & subdir.name)
	Next

End Sub




Sub PrintTranscodingInfo (strCodec,strVideoBitrate,strAudioBitrate,strAudioCodec,strChannels,strMux,strSource,strTarget,strScale)
	Wscript.Echo "Video Codec:   " & strCodec 
	Wscript.Echo "Video Bitrate: " & strVideoBitrate 
	Wscript.Echo "Audio Bitrate: " & strAudioBitrate 
	Wscript.Echo "Audio Codec:   " & strAudioCodec 
	Wscript.Echo "Audio Channels:" & strChannels 
	Wscript.Echo "Used Muxer:    " & strMux 
	Wscript.Echo "Input dir:     " & strSource 
	Wscript.Echo "Output dir:    " & strTarget 
	Wscript.Echo "Scale:         " & strScale 
	Wscript.Echo ""
End Sub




Sub TranscodeMPG(strOldName, strNewName)
	' standard transcoding for SD tv to MP4

	' ----------- used configuration --------------
	strCodec        = "h264"       ' h264 mp4v mp1v mp2v FLV1 MSVC 
	strVideoBitrate = "768"        ' video bitrate in kbits
	strAudioBitrate = "128"        ' audio bitrate in kbits
	strAudioCodec   = "mpga" 	   ' mp4a mpga mp2a vorb flac mp3 a52 (=DD or AC3)
	strChannels     = "2"          ' 1 2 ..
	strMux          = "mp4" 	   ' mpeg1 ps ts mp4 avi asf ogg
	strScale        = "1"          ' scaling relative to original resolution. 1 means 100% of original size
	' ----------- log configuration --------------
	Call PrintTranscodingInfo (strCodec,strVideoBitrate,strAudioBitrate,strAudioCodec,strChannels,strMux,strSource,strTarget,strScale)
	
	' ----------- transcode --------------
	wscript.echo "transcoding " & strOldName &  ".."
	strCmd = strVLCDir & "vlc.exe --verbose 1 --extraintf logger --logfile=" & strNewName & ".log -I dummy """ & strOldName & """ :sout=#transcode{vcodec=" & strCodec & ",vb=" & strVideoBitrate & ",scale=" & strScale & ",acodec=" & strAudioCodec & ",ab=" & strAudioBitrate & ",channels=" & strChannels & "}:duplicate{dst=std{access=file,mux=" & strMux & ",dst=""" & strNewName & """}} vlc://quit"
	
	' print used config (debugging)
	'wscript.echo strCmd 					
	
	' run vlc
	Set objShell = CreateObject("WScript.Shell")
	objShell.Run strCmd, 1, True
	Set objShell = Nothing
	Wscript.Echo Date() &" "& Time() &" " & " done."
	FileTranscoded =FileTranscoded +1
End Sub




Sub TranscodeMP4(strOldName, strNewName)
	' very small & compact target videos (web usage)

	' ----------- used configuration --------------
	strCodec        = "h264"       ' h264 mp4v mp1v mp2v FLV1 MSVC 
	strVideoBitrate = "512"        ' video bitrate in kbits
	strAudioBitrate = "64"         ' audio bitrate in kbits
	strAudioCodec   = "mp4a" 	   ' mp4a mpga mp2a vorb flac mp3 a52 (=DD or AC3)
	strChannels     = "2"          ' 1 2 ..
	strMux          = "mp4" 	   ' mpeg1 ps ts mp4 avi asf ogg
	strScale        = "0.5"        ' scaling relative to original resolution. 1 means 100% of original size
	' ----------- log configuration --------------
	Call PrintTranscodingInfo (strCodec,strVideoBitrate,strAudioBitrate,strAudioCodec,strChannels,strMux,strSource,strTarget,strScale)
	
	' ----------- transcode --------------
	wscript.echo "transcoding " & strOldName &  ".."
	strCmd = strVLCDir & "vlc.exe --verbose 1 --extraintf logger --logfile=" & strNewName & ".log -I dummy """ & strOldName & """ :sout=#transcode{vcodec=" & strCodec & ",vb=" & strVideoBitrate & ",scale=" & strScale & ",acodec=" & strAudioCodec & ",ab=" & strAudioBitrate & ",channels=" & strChannels & "}:duplicate{dst=std{access=file,mux=" & strMux & ",dst=""" & strNewName & """}} vlc://quit"
	
	' print used config (debugging)
	'wscript.echo strCmd 					
	
	' run vlc
	Set objShell = CreateObject("WScript.Shell")
	objShell.Run strCmd, 1, True
	Set objShell = Nothing
	Wscript.Echo Date() &" "& Time() &" " & " done."
	FileTranscoded =FileTranscoded +1
End Sub



Sub TranscodeAVItoMPG2(strOldName, strNewName)
	' AVI to mpg, resize to 1024x768
	' ----------- used configuration --------------
	strCodec        = "mp2v"       ' h264 mp4v mp1v mp2v FLV1 MSVC 
	strVideoBitrate = "3000"       ' video bitrate in kbits
	strAudioBitrate = "320"        ' audio bitrate in kbits
	strAudioCodec   = "mp2a" 	   ' mp4a mpga mp2a vorb flac mp3 a52 (=DD or AC3)
	strChannels     = "2"          ' 1 2 ..
	strMux          = "ps" 	       ' mpeg1 ps ts mp4 avi asf ogg
	'strScale        = "1.0"        ' scaling relative to original resolution. 1 means 100% of original size
	' ----------- log configuration --------------
	Call PrintTranscodingInfo (strCodec,strVideoBitrate,strAudioBitrate,strAudioCodec,strChannels,strMux,strSource,strTarget,strScale)
	
	' ----------- transcode --------------
	wscript.echo "transcoding " & strOldName &  ".."
	strCmd = strVLCDir & "vlc.exe --verbose 1 --extraintf logger --logfile=" & strNewName & ".log -I dummy """ & strOldName & """ :sout=#transcode{vcodec=" & strCodec & ",vb=" & strVideoBitrate & ",width=1024,height=768" & ",acodec=" & strAudioCodec & ",ab=" & strAudioBitrate & ",channels=" & strChannels & "}:duplicate{dst=std{access=file,mux=" & strMux & ",dst=""" & strNewName & """}} vlc://quit"
	
	' print used config (debugging)
	'wscript.echo strCmd 					
	
	' run vlc
	Set objShell = CreateObject("WScript.Shell")
	objShell.Run strCmd, 1, True
	Set objShell = Nothing
	Wscript.Echo Date() &" "& Time() &" " & " done."
	FileTranscoded =FileTranscoded +1
End Sub


Sub TranscodeAVItoMP4(strOldName, strNewName)
	' AVI to mp4, resize to 1024x768
	' ----------- used configuration --------------
	strCodec        = "h264"       ' h264 mp4v mp1v mp2v FLV1 MSVC 
	strVideoBitrate = "2000"       ' video bitrate in kbits
	strAudioBitrate = "64"         ' audio bitrate in kbits
	strAudioCodec   = "mp4a" 	   ' mp4a mpga mp2a vorb flac mp3 a52 (=DD or AC3)
	strChannels     = "2"          ' 1 2 ..
	strMux          = "mp4" 	   ' mpeg1 ps ts mp4 avi asf ogg
	'strScale        = "1.0"       ' scaling relative to original resolution. 1 means 100% of original size
	' ----------- log configuration --------------
	Call PrintTranscodingInfo (strCodec,strVideoBitrate,strAudioBitrate,strAudioCodec,strChannels,strMux,strSource,strTarget,strScale)
	
	' ----------- transcode --------------
	wscript.echo "transcoding " & strOldName &  ".."
	strCmd = strVLCDir & "vlc.exe --verbose 1 --extraintf logger --logfile=" & strNewName & ".log -I dummy """ & strOldName & """ :sout=#transcode{vcodec=" & strCodec & ",vb=" & strVideoBitrate & ",width=1024,height=768" & ",acodec=" & strAudioCodec & ",ab=" & strAudioBitrate & ",channels=" & strChannels & "}:duplicate{dst=std{access=file,mux=" & strMux & ",dst=""" & strNewName & """}} vlc://quit"
	
	' print used config (debugging)
	wscript.echo strCmd 					
	
	' run vlc
	Set objShell = CreateObject("WScript.Shell")
	objShell.Run strCmd, 1, True
	Set objShell = Nothing
	Wscript.Echo Date() &" "& Time() &" " & " done."
	FileTranscoded =FileTranscoded +1
End Sub




